"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const platform_1 = require("@rxstack/platform");
const _ = require("lodash");
const exceptions_1 = require("@rxstack/exceptions");
const utils_1 = require("./utils");
exports.populate = (schema) => {
    return (event) => __awaiter(this, void 0, void 0, function* () {
        utils_1.restrictToOperations(event.eventType, [platform_1.OperationEventsEnum.POST_EXECUTE]);
        const data = event.getData();
        const result = yield getResult(getIds(data, schema.parentField), schema, event.injector);
        _.isArray(data) ? data.map(value => mapResult(schema, value, result))
            : mapResult(schema, data, result);
    });
};
const getIds = (data, parentField) => {
    const ids = [];
    _.isArray(data) ? data.forEach(value => ids.push(...getItemIds(value, parentField)))
        : ids.push(...getItemIds(data, parentField));
    return [...new Set(ids)];
};
const getItemIds = (value, parentField) => {
    const data = _.get(value, parentField);
    if (typeof data === 'undefined') {
        throw new exceptions_1.BadRequestException(`FieldValue in path ${parentField} is undefined.`);
    }
    return _.isArray(data) ? data : [data];
};
const getResult = (ids, schema, injector) => __awaiter(this, void 0, void 0, function* () {
    const service = injector.get(schema.service);
    const defaultQuery = { where: { [schema.childField]: { '$in': ids } }, limit: ids.length, skip: 0 };
    const method = schema.method || 'findMany';
    const query = schema.query ? _.merge(defaultQuery, schema.query) : defaultQuery;
    return yield service[method](query);
});
const mapResult = (schema, data, result) => {
    const value = _.get(data, schema.parentField);
    const items = _.isArray(value) ? _.filter(result, (i) => value.includes(i[schema.childField])) :
        _.find(result, (i) => i[schema.childField] === value);
    _.set(data, schema.nameAs || schema.parentField, items);
};
//# sourceMappingURL=populate.js.map